# Standard library
import os
from functools import cache
from typing import Literal, Optional

# Local
try:
    from marinate.utils import (
        get_file_path,
        get_cache_key,
        get_cached_output,
        cache_output,
        get_logger,
    )
except ImportError:
    from utils import (
        get_file_path,
        get_cache_key,
        get_cached_output,
        cache_output,
        get_logger,
    )


GLOBAL_CACHE_DIR = None


def set_cache_dir(cache_dir: str):
    global global_cache_dir
    global_cache_dir = cache_dir


def marinate(
    cache_file: Optional[str] = None,
    cache_dir: Optional[str] = None,
    overwrite: bool = False,
    store: Literal["disk", "memory"] = "disk",
    verbose: bool = False,
):
    print_log = get_logger(verbose)

    # Validate store parameter
    if store not in {"disk", "memory"}:
        raise ValueError("Invalid value for 'store'. Must be 'disk' or 'memory'.")

    cache_dir = cache_dir or global_cache_dir

    def decorator(f: callable):
        def decorated(*args, **kwargs) -> any:
            if store == "memory":
                print_log(f"Using in-memory cache for {f.__name__}")
                return cache(f)(*args, **kwargs)

            cache_key = get_cache_key(f, args, kwargs)
            fn_file = f.__globals__["__file__"]
            file_path = get_file_path(fn_file, cache_key, cache_file, cache_dir)

            # Use cache if file exists and isn't invalidated
            if os.path.isfile(file_path) and not overwrite:
                print_log(f"Using cached output for {f.__name__} in {file_path}")
                return get_cached_output(file_path)

            # Execute function and cache output
            output = f(*args, **kwargs)
            cache_output(output, file_path)
            print_log(f"Cached output for {f.__name__} in {file_path}")

            return output

        return decorated

    return decorator



# Standard library
import os
import pickle
import hashlib
from typing import Optional

YELLOW = "\033[93m"
RESET = "\033[0m"
CACHE_DIR = ".marinade"


def get_file_path(
    fn_file: str,
    cache_key: Optional[str] = None,
    cache_file: Optional[str] = None,
    cache_dir: Optional[str] = None,
) -> str:
    file_path = cache_file or cache_key
    file_path = f"{file_path}.pkl" if not file_path.endswith(".pkl") else file_path

    if cache_dir:
        dir_path = cache_dir
    else:
        dir_path = os.path.dirname(fn_file)
        dir_path = os.path.join(dir_path, CACHE_DIR)

    if cache_file:
        return file_path

    if not os.path.exists(dir_path):
        os.makedirs(dir_path)

    return os.path.join(dir_path, file_path)


def get_cache_key(f: callable, *args, **kwargs) -> str:
    f_name = f.__name__
    args_str = "_".join(map(str, args))
    kwargs_str = "_".join(f"{k}={v}" for k, v in sorted(kwargs.items()))
    combined_str = f"{f_name}_{args_str}_{kwargs_str}"
    unique_id = hashlib.md5(combined_str.encode()).hexdigest()
    cache_key = f"{f_name}_{unique_id}"

    return cache_key


def get_cached_output(path: str) -> any:
    with open(path, "rb") as file:
        output = pickle.load(file)
        return output


def cache_output(output: any, path: str):
    with open(path, "wb") as file:
        pickle.dump(output, file)


def get_logger(verbose: bool = False) -> callable:
    def log(s: str):
        if verbose:
            print(f"{YELLOW}marinate | {s}{RESET}")

    return log



import sys

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup
from codecs import open

if sys.version_info[:3] < (3, 0, 0):
    print("Requires Python 3 to run.")
    sys.exit(1)

with open("README.md", encoding="utf-8") as file:
    readme = file.read()

setup(
    name="marinate",
    description="A Python package called marinate",
    long_description=readme,
    long_description_content_type="text/markdown",
    version="v1.0.1",
    packages=["marinate"],
    python_requires=">=3",
    url="https://github.com/shobrook/marinate",
    author="shobrook",
    author_email="shobrookj@gmail.com",
    # classifiers=[],
    install_requires=[],
    keywords=[],
    license="MIT",
)
